require 'mingc'
require 'final'        # for Ruby 1.4

class SWFBase
	protected
	include Mingc

	def initialize(this)
		@this = this
	end

	attr_accessor "this"
end

class SWFRect < SWFBase
	def initialize(minX, maxX, minY, maxY)
		super newSWFRect(minX, maxX, minY, maxY)
	end
end

class SWFShape < SWFBase
	def initialize
		super newSWFShape()

                ObjectSpace::define_finalizer(self) {
			SWFShape_end(@this)
		}
	end

	def setLine(width, r, g, b, a)
		return SWFShape_setLine(@this, width, r, g, b, a)
	end

	def addSolidFill(r, g, b, a)
		return SWFShape_addSolidFill(@this, r, g, b, a)
	end

	def addGradientFill(gradient, flags)
		return SWFShape_addGradientFill(@this, gradient.this, flags)
	end

	def addBitmapFill(bitmap, flags)
		return SWFShape_addBitmapFill(@this, bitmap.this, flags)
	end

	def setLeftFill(fill)
		SWFShape_setLeftFill(@this, fill)
	end

	def setRightFill(fill)
		SWFShape_setRightFill(@this, fill)
	end

	def moveTo(x, y)
		SWFShape_moveTo(@this, x, y)
	end

	def lineTo(x, y)
		SWFShape_lineTo(@this, x, y)
	end

	def lineToRelative(dx, dy)
		SWFShape_lineToRelative(@this, dx, dy)
	end

	def drawRect(rect)
		SWFShape_drawRect(@this, rect.this);
	end

	def curveTo(controlx, controly, anchorx, anchory)
		SWFShape_curveTo(@this, controlx, controly, anchorx, anchory)
	end

	def curveToRelative(controldx, controldy, anchordx, anchordy)
		SWFShape_curveToRelative(@this, controldx, controldy,
									   anchordx, anchordy)
	end
end

class SWFDisplayItem < SWFBase
	def rotate(degrees)
		SWFDisplayItem_rotate(@this, degrees)
	end

	def rotateTo(degrees)
		SWFDisplayItem_rotateTo(@this, degrees)
	end

	def move(x, y)
		SWFDisplayItem_move(@this, x, y)
	end

	def moveTo(x, y)
		SWFDisplayItem_moveTo(@this, x, y)
	end

	def scale(xScale, yScale)
		SWFDisplayItem_scale(@this, xScale, yScale)
	end

	def scaleTo(xScale, yScale)
		SWFDisplayItem_scaleTo(@this, xScale, yScale)
	end

	def skewX(x)
		SWFDisplayItem_skewX(@this, x)
	end

	def skewXTo(x)
		SWFDisplayItem_skewXTo(@this, x);
	end

	def skewY(y)
		SWFDisplayItem_skewY(@this, y)
	end

	def skewYTo(y)
		SWFDisplayItem_skewYTo(@this, y)
	end

	def setName(name)
		SWFDisplayItem_setName(@this, name)
	end

	def setRatio(ratio)
		SWFDisplayItem_setRatio(@this, ratio)
	end

	def setDepth(depth)
		SWFDisplayItem_setDepth(@this, depth)
	end

	def remove
		ObjectSpace::remove_finalizer @proc
		SWFDisplayItem_remove(@this)
	end
end

class SWFMovie < SWFBase
	def initialize
		super newSWFMovie()
	end

	def setRate(rate)
		SWFMovie_setRate(@this, rate)
	end

	def setDimension(x, y)
		SWFMovie_setDimension(@this, x, y)
	end

	def setBackground(r, g, b)
		SWFMovie_setBackground(@this, r, g, b)
	end

	def setNumberOfFrames(totalFrames)
		SWFMovie_setNumberOfFrames(@this, totalFrames)
	end

	def nextFrame
		SWFMovie_nextFrame(@this)
	end

	def add(block)
		return SWFDisplayItem.new(SWFMovie_add(@this, block.this))
	end

	def remove(item)
		SWFMovie_remove(@this, item.this)
	end

	def setSoundStream(sound)
		SWFMovie_setSoundStream(@this, sound.this);
	end

	def simpleOutput
		return SWFMovie_simpleOutput(@this)
	end

	def save(filename)
		SWFMovie_save(@this, filename)
	end
end

class SWFMovieClip < SWFBase
	def initialize
		super newSWFMovieClip()
	end

	def setNumberOfFrames(frames)
		SWFMovieClip_setNumberOfFrames(@this, frames)
	end

	def add(block)
		return SWFDisplayItem.new(SWFMovieClip_add(@this, block.this))
	end

	def remove(item)
		SWFMovieClip_remove(@this, item.this)
	end

	def nextFrame
		SWFMovieClip_nextFrame(@this)
	end

	def getSprite
		return SWFMovieClip_getSprite(@this)
	end
end

class SWFGradient < SWFBase
	def initialize
		super newSWFGradient()
	end

	def addEntry(ratio, r, g, b, a)
		return SWFGradient_addEntry(@this, ratio, r, g, b, a)
	end
end

class SWFMorphShape < SWFBase
	def initialize
		super newSWFMorphShape()
	end

	def getShape1
		return SWFShape.new(getShape1(@this))
	end

	def getShape2
		return SWFShape.new(getShape2(@this))
	end
end

class SWFFont < SWFBase
	def initialize
	end
end

class SWFText < SWFBase
	def initialize
		super newSWFText()
	end

	def setFont(font)
		SWFText_setFont(@this, font.this)
	end

	def setHeight(height)
		SWFText_setHeight(@this, height)
	end

	def setXY(x, y)
		SWFText_setXY(@this, x, y)
	end

	def setColor(r, g, b, a)
		SWFText_setColor(@this, r, g, b, a)
	end

	def addString(s, advance)
		SWFText_addString(@this, s, advance)
	end

	def setSpacing(spacing)
		SWFText_setSpacing(@this, spacing)
	end
end

class SWFSprite < SWFBase
	def initialize
		super newSWFSprite()
	end

	def addBlock(block)
		SWFSprite_addBlock(@this, block.this);
	end
end

class SWFAction < SWFBase
	def initialize
		super newSWFAction()
	end
end
